c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine dump_velocities
#include "paradyn.h"
#include "mpif.h"
      integer istatus(mpi_status_size)
 900  format (i7,3f10.4)
      
      if (node.eq.0) write (6,*) 'Dumping Velocities ...'
      
      if (nvelocities.eq.0) call error('No velocities defined')

      call mpi_barrier(mpi_comm_world,ierror)
      time1 = mpi_wtime()
      
      if (idumpvelflag.eq.0.and.node.eq.0) then
        open (unit=3,file=dumpvelfile)
        close (3,status='delete')
        open (unit=3,file=dumpvelfile)
      endif
      idumpvelflag = 1
      
c write to file

      if (node.ne.0) then
        
        call mpi_recv(ntmp,0,mpi_integer,0,0,
     $       mpi_comm_world,istatus,ierror)
        call mpi_send(tag,nlocal,mpi_integer,0,0,
     $       mpi_comm_world,ierror)
        call mpi_send(v,3*nlocal,mpi_double_precision,0,0,
     $       mpi_comm_world,ierror)
        
      else
        
        write (3,*) 'ITEM: NUMBER OF ATOMS'
        write (3,*) natoms
        write (3,*) 'ITEM: BOUNDARY VELOCITY'
        write (3,*) bndvel(1),bndvel(2),bndvel(3)
        write (3,*) 'ITEM: TIME'
        write (3,*) time_stamp
        write (3,*) 'ITEM: VELOCITIES'
        do i = 1,nlocal
          write (3,900) tag(i),v(1,i),v(2,i),v(3,i)
        enddo
        
        do iproc = 1,nprocs-1
          call mpi_send(ntmp,0,mpi_integer,iproc,0,
     $         mpi_comm_world,ierror)
          call mpi_recv(tagtmp,maxlocal,mpi_integer,iproc,0,
     $         mpi_comm_world,istatus,ierror)
          call mpi_get_count(istatus,mpi_integer,icnt,ierror)
          call mpi_recv(xtmp,3*icnt,mpi_double_precision,iproc,0,
     $         mpi_comm_world,istatus,ierror)
          do i = 1,icnt
            write (3,900) tagtmp(i),xtmp(1,i),xtmp(2,i),xtmp(3,i)
          enddo
        enddo
        
      endif
      
      call mpi_barrier(mpi_comm_world,ierror)
      time2 = mpi_wtime()
      time_io = time_io + time2-time1
      
      return
      end
